function [nb,xb,yb,xl,yl,jl,kl,cn,sn,cur,length]= ...
                                              boundarypoints(ms,nc,nl)

%Function BOUNDARYPOINTS evaluates:
%   The total number of boundary nodal points nb
%   The coordinates xb, yb of the  nodal points of the boundary elements
%   The coordinates xl, yl of the  end points of the boundary elements
%   The consecutive numbers of the ends of each element (jl:origin, jk:end)
%   The direction cosines cn=nx, sn=ny of the elements
%   The length of each element
%==========================================================================
% Evaluate nb=total number of boundary elemnts and formulate
% the arrays xl, yl of the end coordinates
%==========================================================================
nb=0;
for i=1:nc
    if ms(i,1) == 1
        x1=ms(i,3);
        y1=ms(i,4);
        x2=ms(i,5);
        y2=ms(i,6);
        ne=ms(i,2);
        [xl1,yl1]=rectline(x1,y1,x2,y2,ne);
        for j=1:ne
            nb=nb+1;
            xl(nb)=xl1(j);
            yl(nb)=yl1(j);
            cur(nb)=0;
        end
    end
    if ms(i,1) == 2
        x0=ms(i,3);
        y0=ms(i,4);    % coordinates of the center
        phi1=ms(i,5);
        phi2=ms(i,6);  % limit angles of the circular arc
        R=ms(i,7);
        ne=ms(i,2);
        [xl2,yl2,cur2]=circle(x0,y0,phi1,phi2,R,ne);

        for j=1:ne
            nb=nb+1;
            xl(nb)=xl2(j);
            yl(nb)=yl2(j);
            cur(nb)=cur2(j);
        end
    end
end
%==========================================================================
% Evaluate the coordinates xb and yb of the midpoints of boundary elements
% and the one dimensional arrays  jn,kn containing the serial numbers 
% of the element end points
%==========================================================================
angle=zeros(nb,1);
k=1;
for i=1:nb
    if  i == nl(k)
        if k == 1
            NODF=1;
        end
        if k > 1
            NODF=nl(k-1)+1;
        end

        jl(i)=i;
        kl(i)=NODF;
        k=k+1;
    else
        jl(i)=i;
        kl(i)=i+1;
    end
    xb(i)=(xl(kl(i))+xl(jl(i)))/2;
    yb(i)=(yl(kl(i))+yl(jl(i)))/2;
end
%==========================================================================
% Evaluate the direction cosines cn=nx,sn=ny and the length of the elements
%==========================================================================
cn=zeros(1,nb);
sn=zeros(1,nb);
length=zeros(1,nb);
for i=1:nb
    dxl=xl(kl(i))-xl(jl(i));
    dyl=yl(kl(i))-yl(jl(i));
    length(i)=sqrt(dxl^2+dyl^2);
    cn(i)=dyl/length(i);
    sn(i)=-dxl/length(i);
end


